# -*-coding:utf-8 -*- #
# ---------------------------------------------------------------------------
# ProjectName:   test
# FileName:      read_excel.py
# Author:       zhang
# Datetime:     2024/11/5 下午7:33
# Description:
# 
# ---------------------------------------------------------------------------
import openpyxl
from APIAutoTest_v3_1 import log
from APIAutoTest_v3_1.common.read_json import read_json
from APIAutoTest_v3_1.common.read_basic_ini import ReadBasicIni
from APIAutoTest_v3_1.common.read_user_ini import ReadUserIni


class ReadExcel:
    def __init__(self, user):
        # 读取基础配置文件
        self.ini = ReadUserIni(user)
        # 读取json文件
        case_data_path = self.ini.get_file_path("case")
        expect_data_path = self.ini.get_file_path("expect")
        sql_data_path = self.ini.get_file_path("sql")
        self.case_data_dict = read_json(case_data_path)
        self.expect_data_dict = read_json(expect_data_path)
        self.sql_data_dict = read_json(sql_data_path)
        # 读取excel文件路径和工作表名称
        excel_path = self.ini.get_file_path("excel")
        table_name = self.ini.get_table_name("name")
        # 读取excel文件
        try:
            wb = openpyxl.load_workbook(excel_path)
            self.ws = wb[table_name]
        except Exception as e:
            log.error(
                f"获取excel的工作簿或工作表出错，excel的路径为：{excel_path}，工作表名称为：{table_name}，错误的类型为：{type(e)}, 错误的描述为：{e}")
            raise e

    def __get_cell_value(self, row, column):
        # 获取指定单元格的数据，获取到之后去掉前后空格
        log.info(f"开始获取第{row}行，{column}列的数据，使用的方法为__get_cell_value")
        try:
            cell_value = self.ws[column + str(row)].value
        except Exception as e:
            log.error(
                f"获取单元格数据时错误，请检查excel文件或输入的行列数据，列为{column}，行为{row}，错误的类型为{type(e)}，错误信息为{e}")
            raise e
        else:
            # 判断单元格是否为空，非空为True
            if cell_value is None:
                return None
            elif cell_value.strip():
                return cell_value.strip()

    def case_module_name(self, row):
        # 获取b列模块名称
        log.info(f"获取{row}行的模块名称，使用的方法为case_module_name")
        return str(self.__get_cell_value(row, "b"))

    def case_api_name(self, row):
        # 获取c列接口名称
        log.info(f"获取{row}行的接口名称，使用的方法为case_api_name")
        return str(self.__get_cell_value(row, "c"))

    def case_req_method(self, row):
        # 获取f列的请求方法
        log.info(f"获取{row}行的请求方法，使用的方法为case_req_method")
        return self.__get_cell_value(row, "f")

    def case_req_url(self, row):
        # 获取g列的请求url
        # 判断请求的url是否为url，为none返回none，不为none返回拼接的域名
        log.info(f"获取{row}行的请求url，使用的方法为case_req_url")
        path = self.__get_cell_value(row, "g")
        if path:
            return ReadBasicIni().get_host("bpm") + path

    def case_req_mime(self, row):
        # 获取h列的请求的媒体类型并转小写
        log.info(f"获取{row}行的请求的媒体类型，使用的方法为case_req_mime")
        mime_ = self.__get_cell_value(row, "h")
        if mime_:
            return mime_.lower()

    def case_data_test(self, row):
        # 获取i列的请求数据，如果模块名称、接口名称其中一个为none不提取用例数据
        # 获取用例的key、模块名称、接口名称
        log.info(f"获取{row}的请求数据，使用的方法为case_data_test")
        data_key = self.__get_cell_value(row, "i")
        module_name = self.case_module_name(row)
        api_name = self.case_api_name(row)
        if module_name and api_name and data_key:
            # 提取用例数据并返回
            return self.case_data_dict[module_name][api_name][data_key]

    def except_data(self, row):
        # 获取j列的期望数据
        # 获取用例的key、模块名称、接口名称
        log.info(f"获取{row}行的期望数据，使用的方法为except_data")
        expect_data_key = self.__get_cell_value(row, "j")
        module_name = self.case_module_name(row)
        api_name = self.case_api_name(row)
        if module_name and api_name and expect_data_key:
            # 提取期望数据并返回
            return self.expect_data_dict[module_name][api_name][expect_data_key]

    def case_sql_data(self, row):
        # 获取l列的sql语句
        # 获取sql的key、模块名称、接口名称
        log.info(f"获取{row}行的sql语句，使用的方法为case_sql_data")
        sql_data = self.__get_cell_value(row, "l")
        module_name = self.case_module_name(row)
        api_name = self.case_api_name(row)
        if module_name and api_name and sql_data:
            # 提取sql数据并返回
            return self.sql_data_dict[module_name][api_name][sql_data]

    def case_sql_type(self, row):
        # 获取m列的sql类型
        log.info(f"获取{row}行的sql类型，使用的方法为case_sql_type")
        sql_type = self.__get_cell_value(row, "k")
        if sql_type:
            return sql_type.lower()

    def update_sql_key(self, rowq):
        # 获取n列的sql更新key
        log.info(f"获取{rowq}行的sql更新key，使用的方法为update_sql_key")
        return self.__get_cell_value(rowq, "m")

    def get_data(self):
        # 获取excel中的所有数据
        # 返回一个二维列表，列表中存放每行相关数据
        log.info("获取excel中的所有用例数据，使用的方法为get_data")
        # 定义一个空列表，存放所有数据
        case_data = []
        # 循环获取有数据的最大行
        for row in range(2, self.ws.max_row + 1):
            # 根据行号获取每行的测试数据
            get_method = self.case_req_method(row)
            get_url = self.case_req_url(row)
            get_mime = self.case_req_mime(row)
            get_case_data_test = self.case_data_test(row)
            get_except_data = self.except_data(row)
            get_sql_data = self.case_sql_data(row)
            get_sql_type = self.case_sql_type(row)
            get_update_sql_key = self.update_sql_key(row)
            # 过滤空行，然后将数据插入之前的空列表中
            if get_method and get_url and get_except_data:
                case_data.append(
                    [
                        get_method,
                        get_url,
                        get_mime,
                        get_case_data_test,
                        get_except_data,
                        get_sql_data,
                        get_sql_type,
                        get_update_sql_key
                    ]
                )
        else:
            return case_data
